@@ -1,11 +1,11 @@
-# $Id: POE.pm,v 1.48 2000/01/25 19:16:07 rcaputo Exp $
+# $Id: POE.pm,v 1.60 2000/03/23 13:04:55 rcaputo Exp $
 # Copyrights and documentation are after __END__.
 
 package POE;
 
 use vars qw($VERSION);
 
-$VERSION = 0.09;
+$VERSION = 0.10_00;
 
 use strict;
 use Carp;
@@ -281,12 +281,13 @@ package functions as states instead of inline code references.
 
 =item *
 
-piong.perl
+poing.perl
 
-This is a quick and dirty multiple-host icmp ping program.  It
-requires two common vt100 escape codes ("\e[2J" to clear the screen
-and "\e[0;0H\" to home the cursor).  It needs to be run by root, since
-it expects to open a raw socket.
+This is a quick and dirty multiple-host icmp ping program.  Actually,
+it's getting better as creatures feep; it may be useful enough to be a
+separate program.  It requires a vt100 or ANSI terminal.  It needs to
+be run by root, since it expects to open a raw socket for ICMP
+pinging.
 
 I thank Russell Mosemann <mose@ccsn.edu> for the Net::Ping module,
 which I "borrowed" heavily from.  Net::Ping is the route of choice if
@@ -416,6 +417,20 @@ Udp shows how to use UDP sockets with Kernel::select calls.
 
 =item *
 
+watermarks.perl
+
+This program is a cross between wheels.perl (wheel-based server) and
+selects.perl (chargen service).  It creates a chargen service (on port
+32019) that uses watermark events to pause output when the unflushed
+write buffer reaches about 512 bytes.  It resumes spewing chargen
+output when the client finally reads what's waiting for it.
+
+There currently is no program to act as a slow client for it.  Telnet
+or other raw TCP clients may work, especially if the client is running
+at maximum niceness.
+
+=item *
+
 wheels.perl
 
 This program is a basic rot13 server.  It is a basic test of the whole
